%%Software author: 	Ramesh Neelamani
%Modification Date: 	January 31, 2004
%Comments: 		neelsh@rice.edu

% Given the Observation, the noise variance, the blurring function and the
% wavelet system setup (defined in ExptSetup and ForWaRDSetup), this program
% now runs the actual ForWaRD algorithm to perform the deconvolution.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Regularized inversion in ForWaRD. Fourier Processing.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

RegInvfilter = (conj(H(:))) ./ ((abs(H(:)).^2 ) + regparam);
xford = real(ifft(RegInvfilter .* Y(:)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wavelet domian wiener filtering 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Take the RDWT of the xford
[xfordw, xfords] = mrdwtcycle1d(xford,WaveletFiltType1,DecompLevels);

% Calculate the variances of the noise at different levels
tempfilt = real(ifft(RegInvfilter));
[tempw,temps] = mrdwtcycle1d(tempfilt,WaveletFiltType1,DecompLevels);
for lev = 1:DecompLevels
  noisevarw(lev) = matrixnorm(tempw(:,lev),2)*sigma;
end
noisevars = matrixnorm(temps,2)*sigma;
clear tempw temps 

% Step 1 in WDWF: Get the hard thresholded estimate

for lev = 1:DecompLevels
  tempw(:,lev)=HardTh(xfordw(:,lev),ThreshFactor*noisevarw(lev)); %thresh difft wavelet subbands
end
temps = HardTh(xfords,ThreshFactor*noisevars); % thresh scaling subband
clear xfordw xfords
xref = mirdwtcycle1d(tempw,temps.',WaveletFiltType1,DecompLevels);
clear tempw temps

% Use the estimate to perform wavelet domain wiener filtering.

% Calculate the variances of the noise at different levels
[tempw,temps] = mrdwtcycle1d(tempfilt,WaveletFiltType2,DecompLevels);
for lev = 1:DecompLevels
  noisevarw(lev) = matrixnorm(tempw(:,lev),2)*sigma;
end
noisevars = matrixnorm(temps,2)*sigma;
clear tempw temps tempfilt

[xfordw, xfords] = mrdwtcycle1d(xford(:),WaveletFiltType2,DecompLevels);
[xrefw, xrefs] = mrdwtcycle1d(xref(:),WaveletFiltType2,DecompLevels);

% Do the wavelet domain wiener filtering

for lev = 1:DecompLevels
  tempw(:,lev)=xfordw(:,lev) .* ((xrefw(:,lev).^2)./...
	(xrefw(:,lev).^2 + noisevarw(lev)^2)); 
end
temps =xfords .* ((xrefs.^2)./(xrefs.^2 + noisevars^2)); 
clear xford xfordw xfords xrefw xrefs
xward = mirdwtcycle1d(tempw,temps.',WaveletFiltType2,DecompLevels);
clear tempw temps 
