%Software author: 	Ramesh Neelamani
%Modification Date: 	January 31, 2004
%Comments: 		neelsh@rice.edu

% This runs the WVD algorithm. This is not well-suited to deconvolve
% operators with many frequency-domain zeros
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reference: D. L. Donoho, ``Nonlinear solution of linear inverse problems
% by Wavelet-Vaguelette Decomposition,'' Appl. Comput. Harmon. Anal.,
% vol. 2, pp. 101--126, 1995.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

regparamwvd = eps;
RegInvfilter = (conj(H(:))) ./ ((abs(H(:)).^2 ) + regparamwvd);
xfordwvd = real(ifft(RegInvfilter  .* Y(:)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wavelet domian wiener filtering 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Take the RDWT of the xfordwvd
[xfordwvdw, xfordwvds] = mrdwtcycle1d(xfordwvd,WaveletFiltType1,DecompLevels);

% Calculate the variances of the noise at different levels
tempfilt = real(ifft(RegInvfilter));
[tempw,temps] = mrdwtcycle1d(tempfilt,WaveletFiltType1,DecompLevels);
for lev = 1:DecompLevels
  noisevarw(lev) = matrixnorm(tempw(:,lev),2)*sigma;
end
noisevars = matrixnorm(temps,2)*sigma;
clear tempw temps 

% Step 1 in WDWF: Get the hard thresholded estimate

for lev = 1:DecompLevels
  tempw(:,lev)=HardTh(xfordwvdw(:,lev),ThreshFactor*noisevarw(lev)); %thresh difft wavelet subbands
end
temps = HardTh(xfordwvds,ThreshFactor*noisevars); % thresh scaling subband
clear xfordwvdw xfordwvds
xrefwvd = mirdwtcycle1d(tempw,temps.',WaveletFiltType1,DecompLevels);
clear tempw temps

% Use the estimate to perform wavelet domain wiener filtering.

% Calculate the variances of the noise at different levels
[tempw,temps] = mrdwtcycle1d(tempfilt,WaveletFiltType2,DecompLevels);
for lev = 1:DecompLevels
  noisevarw(lev) = matrixnorm(tempw(:,lev),2)*sigma;
end
noisevars = matrixnorm(temps,2)*sigma;
clear tempw temps tempfilt

[xfordwvdw, xfordwvds] = mrdwtcycle1d(xfordwvd(:),WaveletFiltType2,DecompLevels);
[xrefwvdw, xrefwvds] = mrdwtcycle1d(xrefwvd(:),WaveletFiltType2,DecompLevels);

% Do the wavelet domain wiener filtering

for lev = 1:DecompLevels
  tempw(:,lev)=xfordwvdw(:,lev) .* ((xrefwvdw(:,lev).^2)./...
	(xrefwvdw(:,lev).^2 + noisevarw(lev)^2)); 
end
temps =xfordwvds .* ((xrefwvds.^2)./(xrefwvds.^2 + noisevars^2)); 
clear xfordwvd xfordwvdw xfordwvds xrefwvdw xrefwvds
xwvd = mirdwtcycle1d(tempw,temps.',WaveletFiltType2,DecompLevels);
clear tempw temps 
