%This part computes the Wiener deconvolution estimate

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Power spectral density estimation
% This might not be robust for certain images
%
% Reference: A.D. Hillery and R.T. Chin, ``Iterative {W}iener filters
% for image restoration,'' {\em IEEE Trans. Signal Processing}, vol.~39,
% pp.~1892--1899,  Aug. 1991.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
spectplot = 0;
PSD ='Estimate';
if PSD == 'Estimate'
  % Note: We add an offset delta to the PSD estimate to stabilize it
  % at frequencies where the convolution operator response goes to
  % zero.
  delta = 0.0*N;
  Pyy = abs(Y).^2;
  Phh = abs(H).^2;
  sqrdtempnorm = (((norm(y-mean(y))^2  - (N-1)*sigma^2))/(norm(h,1))^2); 
  tempreg = sigma^2*N/(sqrdtempnorm);
  
  % using minkowski's ineq
  Pxx0 = abs( Y .* (conj(H)) ./(Phh + N*tempreg)).^2;
  Pxx = Pxx0;
  for i = 1:Iterations
    M = (conj(H) .* Pxx .* Y) ./ (Phh .* Pxx + N*(sigma^2));% wiener estimate
    PxxY = (Pxx .* N*sigma^2) ./ (Phh.*Pxx + N*(sigma^2));
    Pxx = PxxY + abs(M).^2;
    NormPxx(i) = norm(Pxx(:));
  end
  Pxx = (Pxx+delta*tempreg) *norm(Pxx(:))/norm(Pxx(:) + delta*tempreg);
  Weight = sqrt(Pxx);
  if spectplot 
    if OrgSigAvailable
      figure(10)
      subplot(3,1,1)
      semilogy(Weight.^2)
      junk = axis;
      axis([1 N/2 junk(3) junk(4)])
      title('Estimated Spectrum')
      subplot(3,1,2)
      semilogy(abs(fft(x)).^2)
      axis([1 N/2 junk(3) junk(4)])
      title('Actual Spectrum')
      subplot(3,1,3)
      semilogy(abs(abs(fft(x)).^2 - Weight.^2))
      junk = axis;
      axis([1 N/2 junk(3) junk(4)])
      title('Absolute difference')
    else
      figure(10)
      semilogy(Weight.^2)
      title('Estimated Spectrum') 
      junk = axis;
      axis([1 N/2 junk(3) junk(4)])
    end
  end
else 
  X = fft(x);
  Pxx = abs(X).^2;
  Weight = sqrt(Pxx);
end
clear Pyy Pxx Phh M PxxY i delta 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wiener Estimate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
WienerFilterEst = (conj(H) .* (Weight.^2)) ./ ((abs(H).^2 .* (Weight.^2)) + N*(sigma^2));
xwiener  = real(ifft(WienerFilterEst .* Y));
clear WienerFilterEst 


